package EntityModel::Web::Authentication;
{
  $EntityModel::Web::Authentication::VERSION = '0.003';
}
use EntityModel::Class {
};

=head1 NAME

EntityModel::Web::Authentication - 

=head1 VERSION

version 0.003

=head1 SYNOPSIS


=head1 DESCRIPTION

=cut

1;

__END__

=head1 AUTHOR

Tom Molesworth <cpan@entitymodel.com>

=head1 LICENSE

Copyright Tom Molesworth 2009-2011. Licensed under the same terms as Perl itself.
