use strict;
use warnings;

use Test::More tests => 12;
use_ok('EntityModel::Web::Authentication');
use_ok('EntityModel::Web::Authorization');
use_ok('EntityModel::Web::User');
use_ok('EntityModel::Web::Session');
use_ok('EntityModel::Web::Context');
use_ok('EntityModel::Web::Page::Content');
use_ok('EntityModel::Web::Page');
use_ok('EntityModel::Web::Site');
use_ok('EntityModel::Web::Header');
use_ok('EntityModel::Web::Request');
use_ok('EntityModel::Web::Response');
use_ok('EntityModel::Web');
