package EntityModel::Web::Response;
{
  $EntityModel::Web::Response::VERSION = '0.003';
}
use EntityModel::Class {
	page			=> { type => 'EntityModel::Web::Page' },
	request			=> { type => 'EntityModel::Web::Request' },
	context			=> { type => 'EntityModel::Web::Context' },
};

=head1 NAME

EntityModel::Web::Response - handle response to web request

=head1 VERSION

version 0.003

=head1 SYNOPSIS

=head1 DESCRIPTION

=cut

=head1 METHODS

=cut

1;

__END__

=head1 AUTHOR

Tom Molesworth <cpan@entitymodel.com>

=head1 LICENSE

Copyright Tom Molesworth 2009-2011. Licensed under the same terms as Perl itself.
